# =====================================
# generator=datazen
# version=1.12.1
# hash=590d398349b626cdf55efaac20ebb3f7
# =====================================

"""
vmklib - Useful defaults and other package metadata.
"""

DESCRIPTION = "Simplify project workflows by standardizing use of GNU Make."
PKG_NAME = "vmklib"
VERSION = "0.7.6"
