# =====================================
# generator=datazen
# version=1.13.3
# hash=025fef3d48d942de1e95d9b4ef27a4ac
# =====================================

"""
svgen - Useful defaults and other package metadata.
"""

DESCRIPTION = "A tool for working with scalable vector graphics."
PKG_NAME = "svgen"
VERSION = "0.2.1"
